@@ -24,6 +24,17 @@
 #include <tiffio.h>
 #include "zfuncs.h"
 
+//  Fotoxx definitions
+
+#define fversion "Fotoxx v.12.01.2"                                        //  this fotoxx version
+#define flicense "Free software - GNU General Public License v.3"
+#define fhomepage "http://kornelix.squarespace.com/fotoxx"
+#define ftranslators                                                                \
+      "Translators: Jie Luo, Justa, Peter Landgren, Alexander Krasnopolsky, \n"     \
+      "   Arthur Kalverboer, André Campos Rodovalho, Doriano Blengino"
+#define fcredits "Software used: GTK, libtiff, ufraw, exiftool"
+#define fcontact "Bug reports: kornelix2@gmail.com"
+
 //  GTK definitions
 
 #define PXB GdkPixbuf
@@ -61,8 +72,9 @@
 #define pixbright(pix) (0.25*(pix)[0]+0.65*(pix)[1]+0.10*(pix)[2])         //  pixel brightness, 0-64K
 #define pixred(pix) (25 * pix[0] / (pixbright(pix)+1))                     //  pixel redness, 0-100%
 #define Nrecentfiles 100                                                   //  most recent image files
-#define max_threads 6                                                      //  max. working threads         v.10.10
+#define max_threads 6                                                      //  max. working threads
 #define maxedits 100                                                       //  max. edits, undo/redo depth
+#define MAXIMAGES 100000                                                   //  max. image files supported
 
 #define maxtagcats 200                                                     //  max tag categories
 #define tagcc  50                                                          //  max cc for one tag or category ID
@@ -88,7 +100,7 @@
 //  externals from zfuncs module
 
 namespace  zfuncs {
-   extern char    zlanguage[8];                                            //  current language lc_RC
+   extern char    zlang[8];                                                //  current language lc_RC
    extern cchar   *F1_help_topic;                                          //  F1 context help topic
    extern char    zicondir[200];                                           //  where application icons live
    extern int     zdialog_busy;                                            //  tracks open zdialog's
@@ -97,8 +109,10 @@ namespace  zfuncs {
 
 namespace  image_navi {                                                    //  image_gallery() data
    extern char    *galleryname;                                            //  directory or file list name
-   extern int     gallerytype;                                             //  1/2 = directory / file list
+   extern int     gallerytype;                                             //  1/2/3 = directory/file-list/metadata
    extern int     nfiles, nimages;                                         //  file counts, total and images
+   extern char    **flist;                                                 //  image file list
+   extern char    **mdlist;                                                //  metadata list
    extern int     xwinW, xwinH;                                            //  image gallery window size
    extern int     thumbfilesize;                                           //  thumbnail file image size
    extern int     thumbsize;                                               //  curr. thumbnail display size
@@ -145,6 +159,8 @@ EX int         Fmenulock;                                                  //  l
 EX int         Fslideshow;                                                 //  slide show mode is active
 EX int         Frecent;                                                    //  start with recent files gallery
 EX int         Fprev;                                                      //  start with previous file
+EX int         Fblank;                                                     //  start with blank window
+EX int         KBzmalloclog;                                               //  KB toggle for zmalloc reporting
 
 EX int         Fautosync;                                                  //  auto file sync is requested
 EX int         Fmansync;                                                   //  manual file sync is requested
@@ -179,7 +195,7 @@ EX char        jpeg_quality[8];                                            //  j
 
 //  files and directories in /home/<user>/.fotoxx/
 
-EX char        topdirk_file[200];                                          //  top directory file              v.11.11
+EX char        topdirk_file[200];                                          //  top directory save file         v.11.11
 EX char        tags_defined_file[200];                                     //  tags defined file
 EX char        search_index_file[200];                                     //  search index file               v.11.02
 EX char        recentfiles_file[200];                                      //  file of recent image files      v.11.09
@@ -191,9 +207,9 @@ EX char        annotations_dirk[200];                                      //  d
 //  fotoxx PXM pixmaps
 
 typedef struct  {                                                          //  PXM pixmap
-   char     wmi[8];
-   int      ww, hh, bpc;                                                   //  width, height, bits per color
-   void     *bmp;                                                          //  uint8*/uint16* (bpc=8/16)
+   char        wmi[8];
+   int         ww, hh, bpc;                                                //  width, height, bits per color
+   void        *bmp;                                                       //  uint8*/uint16* (bpc=8/16)
 }  PXM;
 
 EX PXM         *Fpxm8;                                                     //  input file pixmap, PXM-8
@@ -231,10 +247,15 @@ EX int         Mxposn, Myposn;                                             //  m
 EX int         Mxdown, Mydown, Mxdrag, Mydrag;                             //  mouse drag vector
 EX int         Mdrag;                                                      //  mouse drag underway
 EX int         Mcapture;                                                   //  mouse captured by edit function
+EX int         Fmousemain;                                                 //  flag, mouse acts on main window
 
 EX int         KBcapture;                                                  //  KB key captured by edit function
 EX int         KBkey;                                                      //  active keyboard key
 
+EX int         KBcontrolkey;                                               //  keyboard key states, 1 = down
+EX int         KBshiftkey;
+EX int         KB_A_key;
+
 EX int         Ntoplines, Nptoplines;                                      //  lines on top of image in window
 EX int         toplinex1[4], topliney1[4], toplinex2[4], topliney2[4];
 EX int         ptoplinex1[4], ptopliney1[4], ptoplinex2[4], ptopliney2[4];
@@ -252,12 +273,14 @@ struct toptext_t {                                                         //  l
 EX toptext_t   toptext[100];                                               //  text strings written on main window
 EX int         Ntoptext;                                                   //  current count
 
-EX zdialog     *zdedittags;                                                //  edit tags zdialog
-EX zdialog     *zdexifview;                                                //  view EXIF/IPTC data
-EX zdialog     *zdexifedit;                                                //  edit EXIF/IPTC data
-EX zdialog     *zdeditcctext;                                              //  edit IPTC caption and EXIF user comments
-EX zdialog     *zdrename;                                                  //  rename file zdialog
-EX zdialog     *zdsela;                                                    //  select area dialog
+struct topcircle_t {                                                       //  list of red circles that are
+   int      px, py;                                                        //    painted over image whenever
+   int      radius;                                                        //      the main window is updated
+   int      color;                                                         //  1/2/3 = white/black/red
+};
+#define maxtopcircles 100                                                  //  max. number of circles
+EX topcircle_t    topcircles[100];                                         //  circles written on main window
+EX int            Ntopcircles;                                             //  current count
 
 //  select area data
 
@@ -267,13 +290,16 @@ EX int         sa_geom1, sa_geom2;
 EX uint16      sa_endpx[10000], sa_endpy[10000];                           //  last pixel drawn per seqence no.
 EX int         sa_thresh;                                                  //  mouse pixel distance threshold
 EX int         sa_mouseradius;                                             //  mouse selection radius
+EX int         sa_searchrange;                                             //  search range (* mouse radius)
 EX int         sa_mousex, sa_mousey;                                       //  mouse position in image
+EX int         sa_matchcolor;                                              //  search using color match Y/N
 EX double      sa_colormatch;                                              //  color range to match (0.001 to 1.0)
 EX uint16      sa_matchRGB[1000][3];                                       //  match colors, up to 1000
 EX int         sa_Nmatch;                                                  //  count of match colors
-EX int         sa_firewall;                                                //  selected pixels block propagation
+EX int         sa_firewall;                                                //  selected pixels block propagation Y/N
 EX GdkColor    *sa_pixRGB;                                                 //  select area outline color
-EX int         sa_radius, sa_radius2;                                      //  current mouse radius, radius**2
+EX int         sa_radius, sa_radius2;                                      //  current mouse radius, radius **2
+EX int         sa_radius3;                                                 //  search range **2
 EX int         sa_calced;                                                  //  edge calculation done
 EX int         sa_blend;                                                   //  edge blend width
 EX int         sa_minx, sa_maxx;                                           //  enclosing rectangle for area
@@ -284,7 +310,8 @@ EX int         sa_maxstack;
 EX int         sa_Nstack;
 EX int         sa_stat;                                                    //  0/1/2/3 = none/edit/pause/complete
 EX int         sa_mode;                                                    //  1-7 = curr. select area edit method
-EX int         sa_hole;                                                    //  finish failure (outline has a hole)
+EX int         sa_finOK;                                                   //  finish area - success
+EX int         sa_finhole;                                                 //  finish area - area has a hole
 EX int         sa_currseq;                                                 //  current select sequence no.
 EX int         sa_Ncurrseq;                                                //  current sequence pixel count
 EX char        *sa_pixselc;                                                //  maps pixels (re)selected in current cycle
@@ -293,6 +320,7 @@ EX int         sa_Npixel;                                                  //  t
 EX int         sa_fww, sa_fhh;                                             //  valid image dimensions for select area
 EX int         Fshowarea;                                                  //  show select area outline
 EX int         Factivearea;                                                //  select area complete and enabled
+EX int         areanumber;                                                 //  increasing sequential number
 
 //  spline curve data
 
@@ -350,13 +378,14 @@ EX char        *last_session;                                              //  l
 EX int         Ffirsttime;                                                 //  first time startup
 EX int         newfiles;                                                   //  new files found since last synch
 EX int         mwgeom[4];                                                  //  main window position and size
-EX cchar       *tbar_style;                                                //  toolbar style
+EX cchar       *tbar_style;                                                //  toolbar style: icons/text/both
+EX cchar       *startdisplay;                                              //  display: recent/prev/blank/file/dirk
+EX char        *startdirk;                                                 //  start directory (startdisplay=dirk)
+EX char        *startfile;                                                 //  start image file (startdisplay=file)
 EX int         Fwarnoverwrite;                                             //  warn overwrite of original image
-EX char        *lens4_name[4];                                             //  names for 4 lenses
-EX double      lens4_mm[4], lens4_bow[4];                                  //  characteristics for 4 lenses
-EX double      lens_mm, lens_bow;                                          //  current lens characteristics
-EX int         curr_lens;                                                  //  current lens, 0-3
-EX char        lensname[40];                                               //  current lens name
+EX double      lens_mm, lens_bow;                                          //  pano lens mm and bow
+EX double      lens_settings[2];                                           //    "  user settings if no EXIF data
+EX cchar       *zoomratio;                                                 //  zoom ratio, "1.xxxxxx"
 EX int         trimsize[2];                                                //  trim (crop) width, height
 EX char        *trimbuttons[6];                                            //  trim dialog button labels
 EX char        *trimratios[6];                                             //  corresponding aspect ratios
@@ -383,6 +412,15 @@ EX int         warpF_grid[8];                                              //  w
 EX int         ss_funcs[SSNF];                                             //  user preferred transitions
 EX char        *ss_musicfile;                                              //  music file or playlist
 
+//  dialogs with global visibility
+
+EX zdialog     *zdedittags;                                                //  edit tags zdialog
+EX zdialog     *zdexifview;                                                //  view EXIF/IPTC data
+EX zdialog     *zdexifedit;                                                //  edit EXIF/IPTC data
+EX zdialog     *zdeditcctext;                                              //  edit IPTC caption and EXIF user comments
+EX zdialog     *zdrename;                                                  //  rename file zdialog
+EX zdialog     *zdsela;                                                    //  select area dialog
+
 //  GTK functions
 
 int   main(int argc, char * argv[]);                                       //  main program
@@ -409,10 +447,12 @@ void  add_toptext(int ID, int px, int py, cchar *text, cchar *font);       //  p
 void  erase_toptext(int ID);                                               //  remove all text strings with ID
 void  paint_toptext();                                                     //  paint text strings when window repainted
 void  paint_text(int px, int py, cchar *text, cchar *font);                //  paint text on window (window space)
+void  add_topcircle(int px, int py, int radius, int color);                //  paint circles on window (image space)
+void  erase_topcircles();                                                  //  remove all red circles
+void  paint_topcircles();                                                  //  paint red circles when window repainted
 
 typedef  void CBfunc();                                                    //  callback function type
 EX CBfunc   *mouseCBfunc;                                                  //  current mouse handler function
-EX zdialog  *mouse_zd;                                                     //  current dialog holding mouse
 
 void takeMouse(zdialog *zd, CBfunc func, GdkCursor *);                     //  capture mouse for dialog     v.11.03
 void freeMouse();                                                          //  free mouse for main window
@@ -458,32 +498,31 @@ void  m_quit(GtkWidget *, cchar *);                                        //  e
 void  m_manage_collections(GtkWidget *, cchar *);                          //  create and edit image collections
 void  m_move_collections(GtkWidget *, cchar *);                            //  change top image directory
 void  edit_coll_popmenu(GtkWidget *, char *file);                          //  popup menu for editing collections
-void  m_moncheck(GtkWidget *, cchar *);                                    //  check monitor brightness and color
-void  m_mongamma(GtkWidget *, cchar *);                                    //  adjust monitor gamma
-void  m_histogram(GtkWidget *, cchar *);                                   //  start brightness histogram
-void  histogram_paint();                                                   //  update brightness histogram
-void  histogram_destroy();                                                 //  remove histogram window
+void  m_batchconvert(GtkWidget *, cchar *);                                //  convert/resize/export multiple image files
+void  m_conv_raw(GtkWidget *, cchar *);                                    //  convert RAW files to tiff
 void  m_slideshow(GtkWidget *, cchar *);                                   //  enter or leave slideshow mode
 void  slideshow_next(cchar *);                                             //  show prev/next image
+void  m_syncfiles(GtkWidget *, cchar *);                                   //  rebuild thumbnails and search index
+int   syncfiles_func(void *);                                              //  sync files function (in subprocess)
 void  m_show_RGB(GtkWidget *, cchar *);                                    //  show RGB values at mouse click
 void  m_gridlines(GtkWidget *, cchar *);                                   //  set up grid lines
 void  load_grid(int *griddata);                                            //  load grid from saved state
 void  save_grid(int *griddata);                                            //  save grid to saved state
 void  toggle_grid(int action);                                             //  set grid off/on or toggle (0/1/2)
-void  m_lensparms(GtkWidget *, cchar *);                                   //  edit lens parameters
+void  m_burn(GtkWidget *, cchar *);                                        //  burn selected images to CD/DVD
+void  m_email(GtkWidget *, cchar *);                                       //  e-mail selected images
+void  m_moncheck(GtkWidget *, cchar *);                                    //  check monitor brightness and color
+void  m_mongamma(GtkWidget *, cchar *);                                    //  adjust monitor gamma
+void  m_histogram(GtkWidget *, cchar *);                                   //  start brightness histogram
+void  histogram_paint();                                                   //  update brightness histogram
+void  histogram_destroy();                                                 //  remove histogram window
 void  m_lang(GtkWidget *, cchar *);                                        //  change language
 void  m_translate(GtkWidget *, cchar *);                                   //  edit translation
 void  m_menu_launcher(GtkWidget *, cchar *);                               //  make desktop menu entry and launcher
-void  m_conv_raw(GtkWidget *, cchar *);                                    //  convert RAW files to tiff
-void  m_burn(GtkWidget *, cchar *);                                        //  burn selected images to CD/DVD
-void  m_email(GtkWidget *, cchar *);                                       //  e-mail selected images
-void  m_syncfiles(GtkWidget *, cchar *);                                   //  rebuild thumbnails and search index
-int   syncfiles_func(void *);                                              //  sync files function (in subprocess)
-void  m_tbar_style(GtkWidget *, cchar *);                                  //  customize toolbar style dialog
-void  tbar_style_set(cchar *style);                                        //  set toolbar style (initz. and dialog)
+void  m_settings(GtkWidget *, cchar *);                                    //  user settings
 void  m_memory_usage(GtkWidget *, cchar *);                                //  report memory consumption by category
 
-//  Image Information menu functions (EXIF, IPTC, etc.)
+//  info menu functions (EXIF, IPTC, etc.)
 
 void  m_edit_cctext(GtkWidget *, cchar *);                                 //  edit caption and user comments dialog
 void  m_edit_tags(GtkWidget *, cchar *);                                   //  edit tags dialog
@@ -500,9 +539,11 @@ void  info_view(int length);                                               //  v
 void  m_info_edit(GtkWidget *, cchar *);                                   //  add or change EXIF/IPTC data
 void  m_info_delete(GtkWidget *, cchar *);                                 //  delete EXIF/IPTC data
 char ** info_get(cchar *file, cchar **keys, int nkeys);                    //  get EXIF/IPTC data for given key(s)
+char ** info_getN(cchar **files, int NF, cchar **keys, int NK);            //   " " data for given keys and files
 int   info_put(cchar *file, cchar **keys, cchar **text, int nkeys);        //  put EXIF/IPTC data for given key(s)
 int   info_copy(cchar *f1, cchar *f2, cchar **k, cchar **t, int nk);       //  copy EXIF/IPTC data from file to file
 void  m_search_images(GtkWidget *, cchar *);                               //  search tags, comments, captions, files
+void  m_search_metadata(GtkWidget *, cchar *);                             //  search and report image metadata
 
 //  select area menu functions
 
@@ -527,8 +568,7 @@ void  sa_draw_mousefunc();                                                 //  l
 int   sa_nearpix(int mx, int my, int rad, int &npx, int &npy);             //  find nearest line-end
 void  sa_draw_line(int px1, int py1, int px2, int py2);                    //  draw a connected line
 void  sa_draw1pix(int px, int py);                                         //  draw one pixel if not already
-void  sa_color_mousefunc();                                                //  color range select function
-void  sa_radius_mousefunc();                                               //  radius select function
+void  sa_mouse_mousefunc();                                                //  select by radius and color match
 void  sa_nextseq();                                                        //  start next sequence number
 void  sa_unselect_pixels();                                                //  remove current selection
 
@@ -549,8 +589,8 @@ void  sa_edgecalc();                                                       //  c
 
 void  m_rotate(GtkWidget *, cchar *);                                      //  rotate image
 void  m_trim(GtkWidget *, cchar *);                                        //  trim image
+void  m_autotrim(GtkWidget *, cchar *);                                    //  autotrim image after rotate, warp
 void  m_resize(GtkWidget *, cchar *);                                      //  resize image
-void  m_batchresize(GtkWidget *, cchar *);                                 //  resize many images
 void  m_annotate(GtkWidget *, cchar *);                                    //  annotate image
 void  m_flip(GtkWidget *, cchar *);                                        //  flip horizontally or vertically
 void  m_negate(GtkWidget *, cchar *);                                      //  B+W and color negatives
@@ -579,6 +619,7 @@ void  m_sharpen(GtkWidget *, cchar *);                                     //  s
 void  m_denoise(GtkWidget *, cchar *);                                     //  image noise reduction
 void  m_smart_erase(GtkWidget *, const char *);                            //  smart erase object
 void  m_dust(GtkWidget *, const char *);                                   //  remove dust
+void  m_stuckpix(GtkWidget *, const char *);                               //  fix stuck pixels 
 void  m_pixedit(GtkWidget *, cchar *);                                     //  edit pixels
 
 //  image edit functions - art menu
@@ -656,7 +697,6 @@ PXM * f_load(cchar *filespec, int bpc);                                    //  l
 int   f_save(char *outfile, cchar *type, int bpc);                         //  save current image pixmap to a file
 PXM * TIFFread(cchar *filespec);                                           //  read TIFF file, native bpc
 int   TIFFwrite(PXM *pxm, cchar *filespec);                                //  write TIFF file, bpc from pxm
-void  tiffwarninghandler(cchar *mod, cchar *fmt, va_list);                 //  intercepts TIFF lib warnings
 PXM * PXBread(cchar *filespec);                                            //  read using pixbuf library, bpc = 8
 int   PXBwrite(PXM *pxm, cchar *filespec);                                 //  write using pixbuf library, bpc = 8
 
@@ -667,10 +707,21 @@ void  PXM_free(PXM *&pxm);                                                 //  f
 PXM * PXM_copy(PXM *pxm);                                                  //  copy PXM pixmap
 PXM * PXM_copy_area(PXM *pxm, int orgx, int orgy, int ww, int hh);         //  copy section of PXM pixmap
 PXM * PXM_convbpc(PXM *pxm);                                               //  convert from 8/16 to 16/8 bpc
+void  PXM_fixblue(PXM *pxm);                                               //  set blue = 0 pixels to blue = 2
 PXM * PXM_rescale(PXM *pxm, int ww, int hh);                               //  rescale PXM pixmap (ww/hh)
 void  PXM_update(PXM *p16, PXM *p8, int orgx, int orgy, int ww, int hh);   //  copy section of PXM-16 into PXM-8
 PXM * PXM_rotate(PXM *pxm, double angle);                                  //  rotate PXM pixmap
-void  PXM_fixblue(PXM *pxm);                                               //  set blue = 0 pixels to blue = 1
+
+//   image navigation and thumbnail gallery functions
+//   for functions returning char *, caller responsible for zfree()
+
+char * image_gallery(cchar *filez, cchar *action, int Nth = 0,             //  display image gallery window, navigate, 
+       void ufunc(int Nth, int butt) = 0, GtkWidget *parent = 0);          //    do callback for clicked thumbnails
+int image_gallery_position(cchar *file, int Nth);                          //  get relative position of file in gallery
+int image_file_type(cchar *file);                                          //  determine if directory or image file type
+char * image_thumbfile(char *imagefile);                                   //  get thumbnail filespec, create if missing
+GdkPixbuf * image_thumbnail(char *imagefile, int size = 0);                //  get thumbnail pixbuf, create if missing
+char ** image_gallery_getfiles(char *startdir, GtkWidget *parent = 0);     //  select files from gallery window
 
 //  translatable strings used in multiple dialogs
 
@@ -680,6 +731,7 @@ void  PXM_fixblue(PXM *pxm);                                               //  s
 #define Ball ZTX("All")
 #define Bamount ZTX("Amount")
 #define Bapply ZTX("Apply")
+#define Bblack ZTX("Black")
 #define Bblendwidth ZTX("Blend Width")
 #define Bblue ZTX("Blue")
 #define Bbrightness ZTX("Brightness")
@@ -694,6 +746,7 @@ void  PXM_fixblue(PXM *pxm);                                               //  s
 #define Bdarker ZTX("Darker Areas")
 #define Bdelete ZTX("Delete")
 #define Bdisable ZTX("Disable")
+#define Bdiscard ZTX("Discard special gallery list? \n %s")
 #define Bdone ZTX("Done")
 #define Bedit ZTX("Edit")
 #define Benable ZTX("Enable")
@@ -710,7 +763,6 @@ void  PXM_fixblue(PXM *pxm);                                               //  s
 #define Binvert ZTX("Invert")
 #define Blighter ZTX("Lighter Areas")
 #define Blimit ZTX("limit")
-#define BmyMouse ZTX("my mouse")
 #define Bnew ZTX("New")
 #define Bnext ZTX("Next")
 #define BOK ZTX("OK")
@@ -736,12 +788,14 @@ void  PXM_fixblue(PXM *pxm);                                               //  s
 #define Bshow ZTX("Show")
 #define Bstart ZTX("Start")
 #define Bthresh ZTX("Threshold")
+#define Btoomanyfiles ZTX("exceed %d files")
 #define Bundoall ZTX("Undo All")
 #define Bundolast ZTX("Undo Last")
 #define Bundo ZTX("Undo")
 #define Bunfinish ZTX("Unfinish")
 #define Bunselect ZTX("Unselect")
 #define Bview ZTX("View")
+#define Bwhite ZTX("White")
 #define Bwidth ZTX("Width")
 
 
