@@ -39,6 +39,9 @@
 #include <execinfo.h>
 #include <locale.h>
 
+#include <gtk/gtk.h>
+#include <gdk/gdkkeysyms.h>
+
 #define  int8   char                                                       //  number types
 #define  int16  short
 #define  int32  long
@@ -47,9 +50,9 @@
 #define  uint16  unsigned short
 #define  uint32  unsigned long
 #define  uint64  unsigned long long
+#define  dub  double
 #define  uchar  unsigned char
 #define  cchar  const char
-#define  dub  double
 
 #define  wstrerror(err) strerror(WEXITSTATUS(err))                         //  get text status for child process
 
@@ -67,9 +70,9 @@
 
 //  system functions ======================================================
 
-void zappcrash(cchar *pMessage, ...);                                      //  crash with popup message in text window
 void apppause();                                                           //  output message and wait for user
 void apppause(cchar * pMess, ... );                                        //  same, works like printf
+void zappcrash(cchar *pMessage, ...);                                      //  crash with popup message in text window
 void catch_signals();                                                      //  catch signals and do backtrace dump
 void beroot(int argc = 0, char *argv[] = 0);                               //  restart image as root if password OK
 
@@ -112,6 +115,7 @@ int samedirk(cchar *file1, cchar *file2);                                  //  r
 int parsefile(cchar *path, char **dir, char **file, char **ext);           //  parse a filespec
 
 //  measure CPU time spent in a function or code block within a function
+
 extern volatile double   cpu_profile_timer;                                //  internal data tables
 extern volatile double   cpu_profile_table[100];
 extern volatile double   cpu_profile_elapsed;
@@ -212,7 +216,8 @@ cchar * SearchWild(cchar *wpath, int &flag);                               //  w
 
 //  search and sort functions =============================================
 
-int bsearch(int element, int nn, int list[]);                              //  binary search sorted list[nn]
+int bsearch(int seekint, int nn, int list[]);                              //  binary search sorted list[nn]
+int bsearch(char *seekrec, char *allrecs, int recl, int nrecs);            //  binary search sorted records
 
 typedef int HeapSortUcomp(cchar * rec1, cchar * rec2);                     //  return -1/0/+1 if rec1 </=/> rec2
 void HeapSort(int vv[], int nn);                                           //  Heap Sort - integer
@@ -244,7 +249,7 @@ double spline2(double x);                                                  //  r
 #define ZTXmaxcc 4000                                                      //  max. cc per string
 
 int zinitapp(cchar *appname, ...);                                         //  initz. app directories and files   v.4.1
-cchar * get_zinstalloc();                                                  //  get /usr or /usr/local  ...        v.4.1
+cchar * get_zprefix();                                                     //  get /usr or /usr/local  ...        v.4.1
 cchar * get_zuserdir();                                                    //  get /home/user/.appname/
 cchar * get_zdatadir();                                                    //  get install directory
 cchar * get_zdocdir();                                                     //  get document directory             v.3.8
@@ -255,21 +260,14 @@ void showz_changelog();                                                    //  s
 void showz_translations();                                                 //  show TRANSLATIONS file in popup window
 void showz_doctext(cchar *file);                                           //  show text file (or .gz) in a popup window
 void zmake_menu_launcher(cchar *command, cchar *cats, cchar *generic);     //  add desktop menu and launcher      v.4.1
-
-void ZTXinit(cchar *lang);                                                 //  setup for message translation
-cchar *ZTX(cchar *english);                                                //  get translation for English message
-void ZTX_translation_start();                                              //  start online translation
+void showz_html(cchar *url);                                               //  show html via preferred browser  v.2.18
 
 /**************************************************************************
    GTK utility functions
 ***************************************************************************/
 
-#include <gtk/gtk.h>
-#include <gdk/gdkkeysyms.h>
-
-#define  zdcbmax 100                                                       //  max. combo box drop-down list
-
 void zmainloop(int skip = 0);                                              //  do main loop, process menu events
+void zthreadcrash();                                                       //  crash if thread is not main() thread
 
 //  text window print and read utilities
 void wprintx(GtkWidget *Win, int line, cchar *mess, cchar *font = 0);      //  write text to line, optional font
@@ -287,7 +285,10 @@ void wprintp(GtkWidget *Win);                                              //  p
 
 //  functions to simplify building menus, tool bars, status bars
 
-#define G_SIGNAL(window,event,func,arg) g_signal_connect(G_OBJECT(window),event,G_CALLBACK(func),(void *) arg);
+#define G_SIGNAL(window,event,func,arg) \
+        g_signal_connect(G_OBJECT(window),event,G_CALLBACK(func),(void *) arg)
+
+#define  zdcbmax 100                                                             //  max. combo box drop-down list
 
 typedef void mtFunc(GtkWidget *, cchar *mname);                                  //  menu or button response function
 
@@ -394,12 +395,19 @@ void zdialog_save_position(zdialog *zd);                                   //  s
 
 /**************************************************************************/
 
-//   GTK misc. utility functioins
+//  translation functions
+
+void ZTXinit(cchar *lang);                                                 //  setup for message translation
+cchar *ZTX(cchar *english);                                                //  get translation for English message
+void ZTX_translation_start(GtkWidget *parent);                             //  start online translation
 
 //  write text to popup window, shell command to popup window
+
 int write_popup_text(cchar *action, cchar *text = 0, int ww = 0, int hh = 0, GtkWidget *parent = 0);
 int popup_command(cchar *command, int ww = 400, int hh = 300, GtkWidget *parent = 0);
 
+//  popup message dialogs
+
 void zmessageACK(GtkWidget *parent, cchar *pMess, ... );                   //  display message, wait for OK
 void zmessLogACK(GtkWidget *parent, cchar *pMess, ... );                   //  same, with log to STDOUT
 int zmessageYN(GtkWidget *parent, cchar *pMess, ... );                     //  display message, wait for YES or NO
@@ -408,38 +416,29 @@ zdialog * zmessage_post(GtkWidget *parent, int secs, cchar *mess, ...);    //  d
 char * zdialog_text(GtkWidget *parent, cchar * title, cchar * initext);    //  get short text input from user
 int zdialog_choose(cchar *title, GtkWidget *parent, cchar *mess, ...);     //  show message and return choice button
 
-//  file chooser dialogs for 1 file or multiple files
+//  file chooser dialogs for one file or multiple files
+
 char  * zgetfile1(cchar *title, cchar *action, cchar *file, cchar *butt = 0);
 char ** zgetfileN(cchar *title, cchar *action, cchar *file, cchar *butt = 0);
 
-void showz_html(cchar *url);                                               //  show html via preferred browser  v.2.18
+//  print image file
+
+void print_image_paper_setup(GtkWidget *parent);                           //  paper setup for printing an image file
+void print_image_margins_setup(GtkWidget *parent);                         //  setup print margins
+void print_image_file(cchar *imagefile);                                   //  print the image file
+
+//  drag and drop functions
 
 typedef void drag_drop_func(int x, int y, char *text);                     //  user function, get drag_drop text  v.2.19
 void drag_drop_connect(GtkWidget *window, drag_drop_func);                 //  connect window to user function
 
+//  miscellaneous GDK/GTK functions
+
+GdkPixbuf * get_thumbnail(char *fpath, int size);                          //  get sized thumbnail for image file
 GdkCursor * zmakecursor(cchar *iconfile);                                  //  make a cursor from an image file   v.3.7
-
-/**************************************************************************/
-
-//   GDK/GTK image file utility functions
-//   for functions returning char *, caller responsible for zfree()
-
-void print_image_paper_setup();                                            //  paper setup for printing an image file
-void print_image_margins_setup();                                          //  setup print margins
-void print_image_file(cchar *imagefile);                                   //  print the image file
-
+int gdk_window_move_pointer(GdkWindow *, int px, int py);                  //  move the mouse pointer to px, py
 GdkPixbuf * gdk_pixbuf_rotate(GdkPixbuf *, double deg, int alfa = 0);      //  rotate pixbuf through any angle
 
-char * image_gallery(cchar *filez, cchar *action, int Nth = 0,             //  display image gallery window, navigate, 
-       void ufunc(int Nth, int butt) = 0, GtkWidget *parent = 0);          //    do callback for clicked thumbnails
-
-int image_gallery_position(cchar *file, int Nth);                          //  get relative position of file in gallery
-int image_file_type(cchar *file);                                          //  determine if directory or image file type
-char * image_thumbfile(char *imagefile);                                   //  get thumbnail filespec, create if missing
-GdkPixbuf * image_thumbnail(char *imagefile, int size = 0);                //  get thumbnail pixbuf, create if missing
-
-char ** image_gallery_getfiles(char *startdir, GtkWidget *parent = 0);     //  select files from gallery window
-
 /**************************************************************************/
 
 //  parameter management functions
